/**
 *  
 *  @header AgoraChatErrorCode.h
 *  @abstract The SDK defined error type.
 *  @author Hyphenate
 *  @version 3.00
 */

#ifndef AgoraChatErrorCode_h
#define AgoraChatErrorCode_h

typedef NS_ENUM(NSInteger, AgoraChatErrorCode) {
    
    AgoraChatErrorNoError = 0,                      /**  no error */

    AgoraChatErrorGeneral = 1,                      /**  A general error. */
    AgoraChatErrorNetworkUnavailable,               /**  The network is unavailable. */
    AgoraChatErrorDatabaseOperationFailed,          /**  The database operation fails. */
    AgoraChatErrorExceedServiceLimit,               /**  You have exceeded the service limit. */
    AgoraChatErrorServiceArrearages,                /**  This error code is deprecated. Your service charge is overdue.*/
    AgoraChatErrorPushReportActionFailed,            /**  Report push action failed*/
    
    AgoraChatErrorInvalidAppkey = 100,              /**  The App key is invalid. */
    AgoraChatErrorInvalidUsername,                  /**  The user ID is invalid. */
    AgoraChatErrorInvalidPassword,                  /**  The password is invalid. */
    AgoraChatErrorInvalidURL,                       /**  The URL is invalid. */
    AgoraChatErrorInvalidToken,                     /**  The token is invalid. */
    AgoraChatErrorUsernameTooLong,                  /**  The username is too long. */
    AgoraChatErrorChannelSyncNotOpen,               /**  The Channel message sync is not enabled. */
    AgoraChatErrorInvalidConversation,              /**  Invalid Conversation. */
    AgoraChatErrorTokenExpire,                      /**  The token has expired. */
    AgoraChatErrorTokeWillExpire,                   /**  The Token will Expire. */
    
    AgoraChatErrorInvalidParam = 110,               /**  Parameters is invalid. */
    AgoraChatErrorOperationUnsupported,             /**  unsupported operation. */
    AgoraChatErrorQueryParamReachesLimit,           /**  Query param reaches limit. */
    
    AgoraChatErrorUserAlreadyLoginSame = 200,       /**  The user is already logged in. */
    AgoraChatErrorUserNotLogin,                     /**  The user is not logged in. */
    AgoraChatErrorUserAuthenticationFailed,         /**  The user authentication fails. */
    AgoraChatErrorUserAlreadyExist,                 /**  The user already exists. */
    AgoraChatErrorUserNotFound,                     /**  The user is not found. */
    AgoraChatErrorUserIllegalArgument,              /**  Invalid argument. */
    AgoraChatErrorUserLoginOnAnotherDevice,         /**  The user has logged in from another device. */
    AgoraChatErrorUserRemoved,                      /**  The user was removed from the server. */
    AgoraChatErrorUserRegisterFailed,               /**  Registration fails. */
    AgoraChatErrorUpdateApnsConfigsFailed,          /**  Fails to configure the Apple push notification.*/
    AgoraChatErrorUserPermissionDenied,             /**  The user has no operation permission. */
    AgoraChatErrorUserBindDeviceTokenFailed,        /**  The bind device token failed. */
    AgoraChatErrorUserUnbindDeviceTokenFailed,      /**  Fails to unbind the user with the device token. */
    AgoraChatErrorUserBindAnotherDevice,            /**  The user is bound to another device and automatic login is not allowed.*/
    AgoraChatErrorUserLoginTooManyDevices,          /**  The user is logged on too many devices. By default, a user can log in to the same account on at most four devices. To modify this number or disable multi-device login, contact sales@agora.io. */
    AgoraChatErrorUserMuted,                        /**  The user is muted in the chat group or chatroom. */
    AgoraChatErrorUserKickedByChangePassword,       /**  The user has changed the password. */
    AgoraChatErrorUserKickedByOtherDevice,          /**  The user was kicked off from another device. */
    AgoraChatErrorUserAlreadyLoginAnother,          /**  Another user is already logged in. */
    AgoraChatErrorUserMutedByAdmin,                 /**  The user is muted by the admin. */
    AgoraChatErrorUserDeviceChanged,                /**  User device different with last login, need login again */
    AgoraChatErrorUserNotOnRoster,                  /**  User is not your friend, can not send message */

    AgoraChatErrorServerNotReachable = 300,         /**  The server is not reachable. */
    AgoraChatErrorServerTimeout,                    /**  The server response times out. */
    AgoraChatErrorServerBusy,                       /**  The server is busy. */
    AgoraChatErrorServerUnknownError,               /**  An unknown server error occurs. */
    AgoraChatErrorServerGetDNSConfigFailed,         /**  Fails to retrieve the DNS configurations. */
    AgoraChatErrorServerServingForbidden,           /**  The service is forbidden. */
    AgoraChatErrorServerDecryptionFailed,           /**  Server transfer decryption failure. */
    AgoraChatErrorServerGetRTCConfigFailed,        /**  Can't get rtc config. */
    AgoraChatErrorFileNotFound = 400,               /**  Cannot find the file. */
    AgoraChatErrorFileInvalid,                      /**  The file is invalid. */
    AgoraChatErrorFileUploadFailed,                 /**  Fails to upload the file. */
    AgoraChatErrorFileDownloadFailed,               /**  Fails to download the file. */
    AgoraChatErrorFileDeleteFailed,                 /**  Fails to delete the file. */
    AgoraChatErrorFileTooLarge,                     /**  The file is too large. */
    AgoraChatErrorFileContentImproper,              /**  The file contains improper content. */
    
    
    AgoraChatErrorMessageInvalid = 500,             /**  The message is invalid. */
    AgoraChatErrorMessageIncludeIllegalContent,     /**  The message contains invalid content. */
    AgoraChatErrorMessageTrafficLimit,              /**  Message sending has exceeded the traffic limit. */
    AgoraChatErrorMessageEncryption,                /**  An error occurs with message encryption. */
    AgoraChatErrorMessageRecallTimeLimit,           /**  Message recall has exceeded the time limit. */
    AgoraChatErrorServiceNotEnable,                 /**  The service is not enabled. */
    AgoraChatErrorMessageExpired,                   /**   The message has expired. */
    AgoraChatErrorMessageIllegalWhiteList,          /**   The message fails to be delivered because the user is not in the whitelist.*/
    AgoraChatErrorMessageExternalLogicBlocked,      /**   The message is blocked by the external logic. */
    AgoraChatErrorMessageCurrentLimiting,           /**   The message sending failure by the current limiting. */
    
    AgoraChatErrorGroupInvalidId = 600,             /**  The group ID is invalid. */
    AgoraChatErrorGroupAlreadyJoined,               /**  The user is already in the group. */
    AgoraChatErrorGroupNotJoined,                   /**  The user has not joined the group. */
    AgoraChatErrorGroupPermissionDenied,            /**  The user does not have permission to access the operation. */
    AgoraChatErrorGroupMembersFull,                 /**  The group has reached the maximum member capacity.*/
    AgoraChatErrorGroupNotExist,                    /**  The group does not exist. */
    AgoraChatErrorGroupSharedFileInvalidId,         /**  The shared file ID is invalid. */
    AgoraChatErrorGroupDisabled,                    /**  The is disable. */
    
    AgoraChatErrorChatroomInvalidId = 700,          /**  The chatroom ID is invalid.*/
    AgoraChatErrorChatroomAlreadyJoined,            /**  The user is already in the chatroom. */
    AgoraChatErrorChatroomNotJoined,                /**  The user has not joined the chatroom. */
    AgoraChatErrorChatroomPermissionDenied,         /**  The user does not have operation permission. */
    AgoraChatErrorChatroomMembersFull,              /**  The chatroom has reached the maximum member capacity. */
    AgoraChatErrorChatroomNotExist,                 /**  The chatroom does not exist. */
    
    
    
    AgoraChatErrorUserCountExceed = 900,            /**  The count of users to get userinfo more than 100 */
    AgoraChatErrorUserInfoDataLengthExceed = 901,   /**  The count of The datalength of userinfo to set is too long */

    
    AgoraChatErrorContactAddFaild = 1000,           /**  Add contact faild */
    AgoraChatErrorContactReachLimit = 1001,    /**  The number of inviter's friends has reached the maximum */
    AgoraChatErrorContactReachLimitPeer = 1002,    /**  The number of invitee's friends has reached the maximum */

    AgoraChatErrorPresenceParamExceed = 1100, /**  The parms has reached the maximum */
    AgoraChatErrorPresenceCannotSubscribeSelf = 1101, /**  Can not subscribe self */
    
    AgoraChatErrorTranslateParamError = 1110, /**  translation parameter error */
    AgoraChatErrorTranslateServiceNotEnabled = 1111,/**  Translation service unavailable */
    AgoraChatErrorTranslateUsageLimit = 1112, /**  Translation is limited */
    AgoraChatErrorTranslateServiceFail = 1113,/**  Translation failed */
    
    AgoraChatErrorModerationFailed = 1200,           /**  The service is forbidden. */
    AgoraChatErrorThirdServiceFailed = 1299,                /**  The service is forbidden. */
    
    AgoraChatErrorReactionReachLimit = 1300,   /**  The number of invitee's friends has reached the maximum */
    AgoraChatErrorReactionHasBeenOperated = 1301,  /**  The number of invitee's friends has reached the maximum */
    AgoraChatErrorReactionOperationIsIllegal = 1302,   /**  The number of invitee's friends has reached the maximum */
    AgoraChatErrorThreadNotExist = 1400,/**  Subsection does not exist */
    AgoraChatErrorThreadAlreadyExist = 1401, /**  Subsection already exists */
    AgoraChatErrorThreadCreateMessageIllegal = 1402,/**  Illegal message to create subarea */
    
    AgoraChatErrorPushBindFailed = 1501,/**  Failed to bind account with push function */
    AgoraChatErrorPushUnBindFailed = 1502,/**  The push function failed to unbind the account */
};

#endif
