/**
 *  
 *  @header AgoraChatMultiDevicesDelegate.h
 *  @abstract This protocol defined the callbacks of Multi-device
 *  @author Hyphenate
 *  @version 3.00
 */

#import <Foundation/Foundation.h>

/**
 *  
 *  Multi-device event types.
 * 
 *  This enumeration takes user A logged into both DeviceA1 and DeviceA2 as an example to illustrate the various multi-device event types and when these events are triggered.
 */
typedef NS_ENUM(NSInteger, AgoraChatMultiDevicesEvent) {
    AgoraChatMultiDevicesEventUnknow = -1,         /**  Default. */
    AgoraChatMultiDevicesEventContactRemove = 2,    /**  If user A deletes a contact on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventContactAccept = 3,    /**  If user A accepts a contact invitation on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventContactDecline = 4,   /**  If user A declines a contact invitation on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventContactBan = 5,       /**  If user A adds another user into the blocklist on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventContactAllow = 6,     /**  If user A removes a user from the blocklist on DeviceA1, this event is triggered on DeviceA2. */
    
    AgoraChatMultiDevicesEventGroupCreate = 10,     /**  If user A creates a chat group on Device A1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupDestroy = 11,    /**  If user A destroys a chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupJoin = 12,       /**  If user A joins a chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupLeave = 13,      /**  If user A leaves a chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupApply = 14,      /**  If user A requests to join a chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupApplyAccept = 15,    /**  If user A receives another user's request to join the chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupApplyDecline = 16,   /**  If user A declines another user's request to join the chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupInvite = 17,     /**  If user A invites other users to join the chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupInviteAccept = 18,   /**  If user A accepts another user's group invitation on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupInviteDecline = 19,  /**  If user A declines another user's group invitation on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupKick = 20,       /**  If user A removes other users from a chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupBan = 21,        /**  If user A is added to the blocklist on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupAllow = 22,      /**  If user A removes other users from a chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupBlock = 23,      /**  If user A blocks messages from a chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupUnBlock = 24,    /**  If user A unblocks messages from a chat group on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupAssignOwner = 25,    /**  If user A assigns a group owner on DeviceA1, this event is triggered on DeviceA2.*/
    AgoraChatMultiDevicesEventGroupAddAdmin = 26,   /**  If user A adds a group admin on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupRemoveAdmin = 27,    /**  If user A removes a group admin on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupAddMute = 28,    /**  If user A mutes other group members on DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupRemoveMute = 29,     /**  If user A unmutes other group members in DeviceA1, this event is triggered on DeviceA2. */
    
    AgoraChatMultiDevicesEventGroupAddWhiteList = 30,  /**  If user A Added other group member to the whitelist in DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupRemoveWhiteList = 31,   /**  If user A removes other member from the whitelist in DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupAllBan = 32,    /**  If user A mutes all group members in DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupRemoveAllBan = 33,  /**  If user A unmutes all group members in DeviceA1, this event is triggered on DeviceA2. */
    AgoraChatMultiDevicesEventGroupDisabled = 34,  /**  If user joined group is disabled, all devices of user A receive the event. */
    AgoraChatMultiDevicesEventGroupAble = 35,  /**  If user joined group cancel disabled, all devices of user A receive the event. */

    
    AgoraChatMultiDevicesEventChatThreadCreate = 40, /**   User A creates an event in the sub-area of device A1, and other devices logged in to this account will receive this event */
    AgoraChatMultiDevicesEventChatThreadDestroy = 41,/**   User A destroys the event in the sub-area of device A1, and all other devices logged in to this account will receive this event */
    AgoraChatMultiDevicesEventChatThreadJoin = 42,/**  User A joins the event in the sub-area of device A1, and other devices logged in to this account will receive this event*/
    AgoraChatMultiDevicesEventChatThreadLeave = 43,/**  User A leaves the event in the sub-area of device A1, and other devices logged in to this account will receive this event */
    AgoraChatMultiDevicesEventChatThreadUpdate = 44,/**  User A updates the event in the sub-area of device A1, and other devices logged in to this account will receive this event */
    AgoraChatMultiDevicesEventChatThreadKick = 45,/**  User A is kicked in the sub-area of device A1, and all other devices that log in to this account will receive this event */
};

@protocol AgoraChatMultiDevicesDelegate <NSObject>

@optional

/**
 *  
 *  The multi-device event callback of contact.
 *
 *  @param aEvent       The event type.
 *  @param aUsername    The username.
 *  @param aExt         The extended Information.
 */
- (void)multiDevicesContactEventDidReceive:(AgoraChatMultiDevicesEvent)aEvent
                                  username:(NSString * _Nonnull)aUsername
                                       ext:(NSString * _Nullable)aExt;

/**
 *  
 *  The multi-device event callback of group.
 *
 *  @param aEvent       The event type.
 *  @param aGroupId     The group ID.
 *  @param aExt         The extended Information.
 */
- (void)multiDevicesGroupEventDidReceive:(AgoraChatMultiDevicesEvent)aEvent
                                 groupId:(NSString * _Nonnull)aGroupId
                                     ext:(id _Nullable)aExt;
/*!
 *  
 *  The multi-device event callback of thread
 *
 *  @param aEvent       Event type
 *  @param aThreadId    subregion id
 *  @param aExt         Extended Information
 */

- (void)multiDevicesChatThreadEventDidReceive:(AgoraChatMultiDevicesEvent)aEvent
                                 threadId:(NSString * _Nonnull)aThreadId
                                     ext:(id _Nullable)aExt;

/*!
 *  
 *  Callback to other devices after setting Do Not Disturb for a single session after enabling multiple devices
 *
 *  @param aEvent       Event type
 *  @param undisturbData         Extended Information
 */
- (void)multiDevicesUndisturbEventNotifyFormOtherDeviceData:(NSString *_Nullable)undisturbData;

@end
