//
//  AgoraChatPresence.h
//  AgoraChat
//
//  Created by lixiaoming on 2022/1/14.
//  Copyright © 2022 easemob.com. All rights reserved.
//

#import <Foundation/Foundation.h>

/*!
 *  
 * The presence details, including the platform used by the publisher's current online device and the current presence state.
 */
@interface AgoraChatPresenceStatusDetail : NSObject
/*!
 *  
 * The platform used by the current online device of the publisher, which can be "ios", "android", "linux", "windows", or "webim".
 */
@property (nonatomic,strong) NSString* _Nonnull device;
/*!
 *  
 * The current presence state of the publisher.
 */
@property (nonatomic) NSInteger status;
@end

/*!
 *  
 *  The presence property class that contains presence properties, including the publisher's user ID and current presence state, and the platform used by the online device, as well as the presence's extension information, update time, and subscription expiration time.
 */
@interface AgoraChatPresence: NSObject
/*!
 *  
 *  The user ID of the presence publisher.
 */
@property (nonatomic,strong) NSString*_Nonnull publisher;

/*!
 *  
 * The details of the current presence state.
 */
@property (nonatomic,strong) NSArray<AgoraChatPresenceStatusDetail*>*_Nullable statusDetails;

/*!
 *  
 *  The presence description information.
 */
@property (nonatomic) NSString*_Nullable statusDescription;

/*!
 *  
 *  The presence update time, which is generated by the server.
 */
@property (nonatomic) NSInteger lastTime;

/*!
 *  
 *  The expiration time of the presence subscription.
 */
@property (nonatomic) NSInteger expirytime;
@end;

